@@ -2004,15 +2004,20 @@ int hwmon_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   hwmon_ctx->enabled = false;
 
+  #if !defined (WITH_HWMON)
+  return 0;
+  #endif // WITH_HWMON
+
   if (user_options->example_hashes    == true) return 0;
   if (user_options->keyspace          == true) return 0;
   if (user_options->left              == true) return 0;
   if (user_options->opencl_info       == true) return 0;
+  if (user_options->progress_only     == true) return 0;
   if (user_options->show              == true) return 0;
+  if (user_options->speed_only        == true) return 0;
   if (user_options->stdout_flag       == true) return 0;
   if (user_options->usage             == true) return 0;
   if (user_options->version           == true) return 0;
-  if (user_options->gpu_temp_disable  == true) return 0;
 
   hwmon_ctx->hm_device = (hm_attrs_t *) hccalloc (DEVICES_MAX, sizeof (hm_attrs_t));
 
