@@ -41,7 +41,7 @@ static const char *const USAGE_BIG[] =
   "     --keep-guessing            |      | Keep guessing the hash after it has been cracked     |",
   "     --self-test-disable        |      | Disable self-test functionality on startup           |",
   "     --loopback                 |      | Add new plains to induct directory                   |",
-  "     --markov-hcstat            | File | Specify hcstat file to use                           | --markov-hc=my.hcstat",
+  "     --markov-hcstat2           | File | Specify hcstat2 file to use                          | --markov-hcstat2=my.hcstat2",
   "     --markov-disable           |      | Disables markov-chains, emulates classic brute-force |",
   "     --markov-classic           |      | Enables classic markov-chains, no per-position       |",
   " -t, --markov-threshold         | Num  | Threshold X when to stop accepting new markov-chains | -t 50",
@@ -63,7 +63,7 @@ static const char *const USAGE_BIG[] =
   "     --remove                   |      | Enable removal of hashes once they are cracked       |",
   "     --remove-timer             | Num  | Update input hash file each X seconds                | --remove-timer=30",
   "     --potfile-disable          |      | Do not write potfile                                 |",
-  "     --potfile-path             | Dir  | Specific path to potfile                             | --potfile-path=my.pot",
+  "     --potfile-path             | File | Specific path to potfile                             | --potfile-path=my.pot",
   "     --encoding-from            | Code | Force internal wordlist encoding from X              | --encoding-from=iso-8859-15",
   "     --encoding-to              | Code | Force internal wordlist encoding to X                | --encoding-to=utf-32le",
   "     --debug-mode               | Num  | Defines the debug mode (hybrid only by using rules)  | --debug-mode=4",
@@ -115,6 +115,17 @@ static const char *const USAGE_BIG[] =
   " -i, --increment                |      | Enable mask increment mode                           |",
   "     --increment-min            | Num  | Start mask incrementing at X                         | --increment-min=4",
   "     --increment-max            | Num  | Stop mask incrementing at X                          | --increment-max=8",
+  " -S, --slow-candidates          |      | Enable slower (but advanced) candidate generators    |",
+  #ifdef WITH_BRAIN
+  "     --brain-server             |      | Enable brain server                                  |",
+  " -z, --brain-client             |      | Enable brain client, activates -S                    |",
+  "     --brain-client-features    | Num  | Define brain client features, see below              | --brain-client-features=3",
+  "     --brain-host               | Str  | Brain server host (IP or domain)                     | --brain-host=127.0.0.1",
+  "     --brain-port               | Port | Brain server port                                    | --brain-port=13743",
+  "     --brain-password           | Str  | Brain server authentication password                 | --brain-password=bZfhCvGUSjRq",
+  "     --brain-session            | Hex  | Overrides automatically calculated brain session     | --brain-session=0x2ae611db",
+  "     --brain-session-whitelist  | Hex  | Allow given sessions only, separated with commas     | --brain-session-whitelist=0x2ae611db",
+  #endif
   "",
   "- [ Hash modes ] -",
   "",
@@ -124,11 +135,18 @@ static const char *const USAGE_BIG[] =
   "      0 | MD5                                              | Raw Hash",
   "   5100 | Half MD5                                         | Raw Hash",
   "    100 | SHA1                                             | Raw Hash",
-  "   1300 | SHA-224                                          | Raw Hash",
-  "   1400 | SHA-256                                          | Raw Hash",
-  "  10800 | SHA-384                                          | Raw Hash",
-  "   1700 | SHA-512                                          | Raw Hash",
-  "   5000 | SHA-3 (Keccak)                                   | Raw Hash",
+  "   1300 | SHA2-224                                         | Raw Hash",
+  "   1400 | SHA2-256                                         | Raw Hash",
+  "  10800 | SHA2-384                                         | Raw Hash",
+  "   1700 | SHA2-512                                         | Raw Hash",
+  "  17300 | SHA3-224                                         | Raw Hash",
+  "  17400 | SHA3-256                                         | Raw Hash",
+  "  17500 | SHA3-384                                         | Raw Hash",
+  "  17600 | SHA3-512                                         | Raw Hash",
+  "  17700 | Keccak-224                                       | Raw Hash",
+  "  17800 | Keccak-256                                       | Raw Hash",
+  "  17900 | Keccak-384                                       | Raw Hash",
+  "  18000 | Keccak-512                                       | Raw Hash",
   "    600 | BLAKE2b-512                                      | Raw Hash",
   "  10100 | SipHash                                          | Raw Hash",
   "   6000 | RIPEMD-160                                       | Raw Hash",
@@ -372,8 +390,19 @@ static const char *const USAGE_BIG[] =
   "  15700 | Ethereum Wallet, SCRYPT                          | Password Managers",
   "  16300 | Ethereum Pre-Sale Wallet, PBKDF2-HMAC-SHA256     | Password Managers",
   "  16900 | Ansible Vault                                    | Password Managers",
+  "  18100 | TOTP (HMAC-SHA1)                                 | One-Time Passwords",
   "  99999 | Plaintext                                        | Plaintext",
   "",
+  #ifdef WITH_BRAIN
+  "- [ Brain Client Features ] -",
+  "",
+  "  # | Features",
+  " ===+========",
+  "  1 | Send hashed passwords",
+  "  2 | Send attack positions",
+  "  3 | Send hashed passwords and attack positions",
+  "",
+  #endif
   "- [ Outfile Formats ] -",
   "",
   "  # | Format",
