@@ -14,14 +14,22 @@
 #include "outfile.h"
 #include "user_options.h"
 
-static const char *short_options = "hVvm:a:r:j:k:g:o:t:d:D:n:u:c:p:s:l:1:2:3:4:iIbw:O";
+#ifdef WITH_BRAIN
+#include "brain.h"
+#endif
+
+#ifdef WITH_BRAIN
+static const char *short_options = "hVvm:a:r:j:k:g:o:t:d:D:n:u:c:p:s:l:1:2:3:4:iIbw:OSz";
+#else
+static const char *short_options = "hVvm:a:r:j:k:g:o:t:d:D:n:u:c:p:s:l:1:2:3:4:iIbw:OS";
+#endif
 
 static const struct option long_options[] =
 {
   {"advice-disable",            no_argument,       NULL, IDX_ADVICE_DISABLE},
   {"attack-mode",               required_argument, NULL, IDX_ATTACK_MODE},
-  {"benchmark",                 no_argument,       NULL, IDX_BENCHMARK},
   {"benchmark-all",             no_argument,       NULL, IDX_BENCHMARK_ALL},
+  {"benchmark",                 no_argument,       NULL, IDX_BENCHMARK},
   {"bitmap-max",                required_argument, NULL, IDX_BITMAP_MAX},
   {"bitmap-min",                required_argument, NULL, IDX_BITMAP_MIN},
   {"cpu-affinity",              required_argument, NULL, IDX_CPU_AFFINITY},
@@ -62,7 +70,7 @@ static const struct option long_options[] =
   {"machine-readable",          no_argument,       NULL, IDX_MACHINE_READABLE},
   {"markov-classic",            no_argument,       NULL, IDX_MARKOV_CLASSIC},
   {"markov-disable",            no_argument,       NULL, IDX_MARKOV_DISABLE},
-  {"markov-hcstat",             required_argument, NULL, IDX_MARKOV_HCSTAT},
+  {"markov-hcstat2",            required_argument, NULL, IDX_MARKOV_HCSTAT2},
   {"markov-threshold",          required_argument, NULL, IDX_MARKOV_THRESHOLD},
   {"nonce-error-corrections",   required_argument, NULL, IDX_NONCE_ERROR_CORRECTIONS},
   {"nvidia-spin-damp",          required_argument, NULL, IDX_NVIDIA_SPIN_DAMP},
@@ -77,9 +85,9 @@ static const struct option long_options[] =
   {"outfile-check-timer",       required_argument, NULL, IDX_OUTFILE_CHECK_TIMER},
   {"outfile-format",            required_argument, NULL, IDX_OUTFILE_FORMAT},
   {"outfile",                   required_argument, NULL, IDX_OUTFILE},
-  {"wordlist-autohex-disable",  no_argument,       NULL, IDX_WORDLIST_AUTOHEX_DISABLE},
   {"potfile-disable",           no_argument,       NULL, IDX_POTFILE_DISABLE},
   {"potfile-path",              required_argument, NULL, IDX_POTFILE_PATH},
+  {"progress-only",             no_argument,       NULL, IDX_PROGRESS_ONLY},
   {"quiet",                     no_argument,       NULL, IDX_QUIET},
   {"remove",                    no_argument,       NULL, IDX_REMOVE},
   {"remove-timer",              required_argument, NULL, IDX_REMOVE_TIMER},
@@ -91,24 +99,35 @@ static const struct option long_options[] =
   {"rules-file",                required_argument, NULL, IDX_RP_FILE},
   {"runtime",                   required_argument, NULL, IDX_RUNTIME},
   {"scrypt-tmto",               required_argument, NULL, IDX_SCRYPT_TMTO},
-  {"self-test-disable",         no_argument,       NULL, IDX_SELF_TEST_DISABLE},
   {"segment-size",              required_argument, NULL, IDX_SEGMENT_SIZE},
+  {"self-test-disable",         no_argument,       NULL, IDX_SELF_TEST_DISABLE},
   {"separator",                 required_argument, NULL, IDX_SEPARATOR},
   {"seperator",                 required_argument, NULL, IDX_SEPARATOR},
   {"session",                   required_argument, NULL, IDX_SESSION},
   {"show",                      no_argument,       NULL, IDX_SHOW},
   {"skip",                      required_argument, NULL, IDX_SKIP},
+  {"slow-candidates",           no_argument,       NULL, IDX_SLOW_CANDIDATES},
+  {"speed-only",                no_argument,       NULL, IDX_SPEED_ONLY},
   {"status",                    no_argument,       NULL, IDX_STATUS},
   {"status-timer",              required_argument, NULL, IDX_STATUS_TIMER},
   {"stdout",                    no_argument,       NULL, IDX_STDOUT_FLAG},
-  {"speed-only",                no_argument,       NULL, IDX_SPEED_ONLY},
-  {"progress-only",             no_argument,       NULL, IDX_PROGRESS_ONLY},
   {"truecrypt-keyfiles",        required_argument, NULL, IDX_TRUECRYPT_KEYFILES},
   {"username",                  no_argument,       NULL, IDX_USERNAME},
   {"veracrypt-keyfiles",        required_argument, NULL, IDX_VERACRYPT_KEYFILES},
   {"veracrypt-pim",             required_argument, NULL, IDX_VERACRYPT_PIM},
   {"version",                   no_argument,       NULL, IDX_VERSION},
+  {"wordlist-autohex-disable",  no_argument,       NULL, IDX_WORDLIST_AUTOHEX_DISABLE},
   {"workload-profile",          required_argument, NULL, IDX_WORKLOAD_PROFILE},
+  #ifdef WITH_BRAIN
+  {"brain-client",              no_argument,       NULL, IDX_BRAIN_CLIENT},
+  {"brain-client-features",     required_argument, NULL, IDX_BRAIN_CLIENT_FEATURES},
+  {"brain-server",              no_argument,       NULL, IDX_BRAIN_SERVER},
+  {"brain-host",                required_argument, NULL, IDX_BRAIN_HOST},
+  {"brain-port",                required_argument, NULL, IDX_BRAIN_PORT},
+  {"brain-password",            required_argument, NULL, IDX_BRAIN_PASSWORD},
+  {"brain-session",             required_argument, NULL, IDX_BRAIN_SESSION},
+  {"brain-session-whitelist",   required_argument, NULL, IDX_BRAIN_SESSION_WHITELIST},
+  #endif
   {NULL,                        0,                 NULL, 0 }
 };
 
@@ -128,10 +147,19 @@ int user_options_init (hashcat_ctx_t *hashcat_ctx)
 
   user_options->advice_disable            = ADVICE_DISABLE;
   user_options->attack_mode               = ATTACK_MODE;
-  user_options->benchmark                 = BENCHMARK;
   user_options->benchmark_all             = BENCHMARK_ALL;
+  user_options->benchmark                 = BENCHMARK;
   user_options->bitmap_max                = BITMAP_MAX;
   user_options->bitmap_min                = BITMAP_MIN;
+  #ifdef WITH_BRAIN
+  user_options->brain_client              = BRAIN_CLIENT;
+  user_options->brain_client_features     = BRAIN_CLIENT_FEATURES;
+  user_options->brain_host                = NULL;
+  user_options->brain_port                = BRAIN_PORT;
+  user_options->brain_server              = BRAIN_SERVER;
+  user_options->brain_session             = BRAIN_SESSION;
+  user_options->brain_session_whitelist   = NULL;
+  #endif
   user_options->cpu_affinity              = NULL;
   user_options->custom_charset_1          = NULL;
   user_options->custom_charset_2          = NULL;
@@ -139,9 +167,9 @@ int user_options_init (hashcat_ctx_t *hashcat_ctx)
   user_options->custom_charset_4          = NULL;
   user_options->debug_file                = NULL;
   user_options->debug_mode                = DEBUG_MODE;
-  user_options->example_hashes            = EXAMPLE_HASHES;
   user_options->encoding_from             = ENCODING_FROM;
   user_options->encoding_to               = ENCODING_TO;
+  user_options->example_hashes            = EXAMPLE_HASHES;
   user_options->force                     = FORCE;
   user_options->gpu_temp_abort            = GPU_TEMP_ABORT;
   user_options->gpu_temp_disable          = GPU_TEMP_DISABLE;
@@ -154,9 +182,9 @@ int user_options_init (hashcat_ctx_t *hashcat_ctx)
   user_options->increment_max             = INCREMENT_MAX;
   user_options->increment_min             = INCREMENT_MIN;
   user_options->induction_dir             = NULL;
+  user_options->keep_guessing             = KEEP_GUESSING;
   user_options->kernel_accel              = KERNEL_ACCEL;
   user_options->kernel_loops              = KERNEL_LOOPS;
-  user_options->keep_guessing             = KEEP_GUESSING;
   user_options->keyspace                  = KEYSPACE;
   user_options->left                      = LEFT;
   user_options->limit                     = LIMIT;
@@ -165,7 +193,7 @@ int user_options_init (hashcat_ctx_t *hashcat_ctx)
   user_options->machine_readable          = MACHINE_READABLE;
   user_options->markov_classic            = MARKOV_CLASSIC;
   user_options->markov_disable            = MARKOV_DISABLE;
-  user_options->markov_hcstat             = NULL;
+  user_options->markov_hcstat2            = NULL;
   user_options->markov_threshold          = MARKOV_THRESHOLD;
   user_options->nonce_error_corrections   = NONCE_ERROR_CORRECTIONS;
   user_options->nvidia_spin_damp          = NVIDIA_SPIN_DAMP;
@@ -175,7 +203,6 @@ int user_options_init (hashcat_ctx_t *hashcat_ctx)
   user_options->opencl_platforms          = NULL;
   user_options->opencl_vector_width       = OPENCL_VECTOR_WIDTH;
   user_options->optimized_kernel_enable   = OPTIMIZED_KERNEL_ENABLE;
-  user_options->wordlist_autohex_disable  = WORDLIST_AUTOHEX_DISABLE;
   user_options->outfile_autohex           = OUTFILE_AUTOHEX;
   user_options->outfile_check_dir         = NULL;
   user_options->outfile_check_timer       = OUTFILE_CHECK_TIMER;
@@ -183,6 +210,7 @@ int user_options_init (hashcat_ctx_t *hashcat_ctx)
   user_options->outfile                   = NULL;
   user_options->potfile_disable           = POTFILE_DISABLE;
   user_options->potfile_path              = NULL;
+  user_options->progress_only             = PROGRESS_ONLY;
   user_options->quiet                     = QUIET;
   user_options->remove                    = REMOVE;
   user_options->remove_timer              = REMOVE_TIMER;
@@ -198,23 +226,24 @@ int user_options_init (hashcat_ctx_t *hashcat_ctx)
   user_options->rule_buf_r                = RULE_BUF_R;
   user_options->runtime                   = RUNTIME;
   user_options->scrypt_tmto               = SCRYPT_TMTO;
-  user_options->self_test_disable         = SELF_TEST_DISABLE;
   user_options->segment_size              = SEGMENT_SIZE;
+  user_options->self_test_disable         = SELF_TEST_DISABLE;
   user_options->separator                 = SEPARATOR;
   user_options->session                   = PROGNAME;
   user_options->show                      = SHOW;
   user_options->skip                      = SKIP;
+  user_options->slow_candidates           = SLOW_CANDIDATES;
+  user_options->speed_only                = SPEED_ONLY;
   user_options->status                    = STATUS;
   user_options->status_timer              = STATUS_TIMER;
   user_options->stdout_flag               = STDOUT_FLAG;
-  user_options->speed_only                = SPEED_ONLY;
-  user_options->progress_only             = PROGRESS_ONLY;
   user_options->truecrypt_keyfiles        = NULL;
   user_options->usage                     = USAGE;
   user_options->username                  = USERNAME;
   user_options->veracrypt_keyfiles        = NULL;
   user_options->veracrypt_pim             = 0;
   user_options->version                   = VERSION;
+  user_options->wordlist_autohex_disable  = WORDLIST_AUTOHEX_DISABLE;
   user_options->workload_profile          = WORKLOAD_PROFILE;
   user_options->rp_files_cnt              = 0;
   user_options->rp_files                  = (char **) hccalloc (256, sizeof (char *));
@@ -282,6 +311,9 @@ int user_options_getopt (hashcat_ctx_t *hashcat_ctx, int argc, char **argv)
       case IDX_BITMAP_MAX:
       case IDX_INCREMENT_MIN:
       case IDX_INCREMENT_MAX:
+      #ifdef WITH_BRAIN
+      case IDX_BRAIN_PORT:
+      #endif
 
       if (hc_string_is_digit (optarg) == false)
       {
@@ -332,8 +364,10 @@ int user_options_getopt (hashcat_ctx_t *hashcat_ctx, int argc, char **argv)
       case IDX_EXAMPLE_HASHES:           user_options->example_hashes            = true;                            break;
       case IDX_FORCE:                    user_options->force                     = true;                            break;
       case IDX_SELF_TEST_DISABLE:        user_options->self_test_disable         = true;                            break;
-      case IDX_SKIP:                     user_options->skip                      = hc_strtoull (optarg, NULL, 10);  break;
-      case IDX_LIMIT:                    user_options->limit                     = hc_strtoull (optarg, NULL, 10);  break;
+      case IDX_SKIP:                     user_options->skip                      = hc_strtoull (optarg, NULL, 10);
+                                         user_options->skip_chgd                 = true;                            break;
+      case IDX_LIMIT:                    user_options->limit                     = hc_strtoull (optarg, NULL, 10);
+                                         user_options->limit_chgd                = true;                            break;
       case IDX_KEEP_GUESSING:            user_options->keep_guessing             = true;                            break;
       case IDX_KEYSPACE:                 user_options->keyspace                  = true;                            break;
       case IDX_BENCHMARK:                user_options->benchmark                 = true;                            break;
@@ -365,7 +399,7 @@ int user_options_getopt (hashcat_ctx_t *hashcat_ctx, int argc, char **argv)
       case IDX_MARKOV_DISABLE:           user_options->markov_disable            = true;                            break;
       case IDX_MARKOV_CLASSIC:           user_options->markov_classic            = true;                            break;
       case IDX_MARKOV_THRESHOLD:         user_options->markov_threshold          = hc_strtoul (optarg, NULL, 10);   break;
-      case IDX_MARKOV_HCSTAT:            user_options->markov_hcstat             = optarg;                          break;
+      case IDX_MARKOV_HCSTAT2:           user_options->markov_hcstat2            = optarg;                          break;
       case IDX_OUTFILE:                  user_options->outfile                   = optarg;                          break;
       case IDX_OUTFILE_FORMAT:           user_options->outfile_format            = hc_strtoul (optarg, NULL, 10);
                                          user_options->outfile_format_chgd       = true;                            break;
@@ -416,6 +450,20 @@ int user_options_getopt (hashcat_ctx_t *hashcat_ctx, int argc, char **argv)
       case IDX_CUSTOM_CHARSET_2:         user_options->custom_charset_2          = optarg;                          break;
       case IDX_CUSTOM_CHARSET_3:         user_options->custom_charset_3          = optarg;                          break;
       case IDX_CUSTOM_CHARSET_4:         user_options->custom_charset_4          = optarg;                          break;
+      case IDX_SLOW_CANDIDATES:          user_options->slow_candidates           = true;                            break;
+      #ifdef WITH_BRAIN
+      case IDX_BRAIN_CLIENT:             user_options->brain_client              = true;                            break;
+      case IDX_BRAIN_CLIENT_FEATURES:    user_options->brain_client_features     = hc_strtoul (optarg, NULL, 10);   break;
+      case IDX_BRAIN_SERVER:             user_options->brain_server              = true;                            break;
+      case IDX_BRAIN_PASSWORD:           user_options->brain_password            = optarg;
+                                         user_options->brain_password_chgd       = true;                            break;
+      case IDX_BRAIN_HOST:               user_options->brain_host                = optarg;
+                                         user_options->brain_host_chgd           = true;                            break;
+      case IDX_BRAIN_PORT:               user_options->brain_port                = hc_strtoul (optarg, NULL, 10);
+                                         user_options->brain_port_chgd           = true;                            break;
+      case IDX_BRAIN_SESSION:            user_options->brain_session             = hc_strtoul (optarg, NULL, 16);   break;
+      case IDX_BRAIN_SESSION_WHITELIST:  user_options->brain_session_whitelist   = optarg;                          break;
+      #endif
     }
   }
 
@@ -445,18 +493,68 @@ int user_options_sanity (hashcat_ctx_t *hashcat_ctx)
     return -1;
   }
 
-  if ((user_options->attack_mode != ATTACK_MODE_STRAIGHT)
-   && (user_options->attack_mode != ATTACK_MODE_COMBI)
-   && (user_options->attack_mode != ATTACK_MODE_BF)
-   && (user_options->attack_mode != ATTACK_MODE_HYBRID1)
-   && (user_options->attack_mode != ATTACK_MODE_HYBRID2)
-   && (user_options->attack_mode != ATTACK_MODE_NONE))
+  #ifdef WITH_BRAIN
+  if ((user_options->brain_client == true) && (user_options->brain_server == true))
   {
-    event_log_error (hashcat_ctx, "Invalid attack mode (-a) value specified.");
+    event_log_error (hashcat_ctx, "Can not have --brain-client and --brain-server at the same time");
 
     return -1;
   }
 
+  if ((user_options->brain_client_features < 1) && (user_options->brain_client_features > 3))
+  {
+    event_log_error (hashcat_ctx, "Invalid --brain-client-feature argument");
+
+    return -1;
+  }
+
+  if ((user_options->brain_client == true) && (user_options->brain_password_chgd == false))
+  {
+    event_log_error (hashcat_ctx, "Brain clients need to set --brain-password");
+
+    return -1;
+  }
+  #endif
+
+  if (user_options->slow_candidates == true)
+  {
+    if ((user_options->attack_mode != ATTACK_MODE_STRAIGHT)
+     && (user_options->attack_mode != ATTACK_MODE_COMBI)
+     && (user_options->attack_mode != ATTACK_MODE_BF))
+    {
+      event_log_error (hashcat_ctx, "Invalid attack mode (-a) value specified in slow-candidates mode.");
+
+      return -1;
+    }
+  }
+  #ifdef WITH_BRAIN
+  else if (user_options->brain_client == true)
+  {
+    if ((user_options->attack_mode != ATTACK_MODE_STRAIGHT)
+     && (user_options->attack_mode != ATTACK_MODE_COMBI)
+     && (user_options->attack_mode != ATTACK_MODE_BF))
+    {
+      event_log_error (hashcat_ctx, "Invalid attack mode (-a) value specified in brain-client mode.");
+
+      return -1;
+    }
+  }
+  #endif
+  else
+  {
+    if ((user_options->attack_mode != ATTACK_MODE_STRAIGHT)
+     && (user_options->attack_mode != ATTACK_MODE_COMBI)
+     && (user_options->attack_mode != ATTACK_MODE_BF)
+     && (user_options->attack_mode != ATTACK_MODE_HYBRID1)
+     && (user_options->attack_mode != ATTACK_MODE_HYBRID2)
+     && (user_options->attack_mode != ATTACK_MODE_NONE))
+    {
+      event_log_error (hashcat_ctx, "Invalid attack mode (-a) value specified.");
+
+      return -1;
+    }
+  }
+
   if (user_options->hccapx_message_pair_chgd == true)
   {
     if (user_options->remove == true)
@@ -474,6 +572,22 @@ int user_options_sanity (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
+  /*
+  if (user_options->skip_chgd == true && user_options->skip == 0)
+  {
+    event_log_error (hashcat_ctx, "Invalid --skip value specified.");
+
+    return -1;
+  }
+  */
+
+  if (user_options->limit_chgd == true && user_options->limit == 0)
+  {
+    event_log_error (hashcat_ctx, "Invalid --limit value specified.");
+
+    return -1;
+  }
+
   if (user_options->runtime_chgd == true && user_options->runtime == 0)
   {
     event_log_error (hashcat_ctx, "Invalid --runtime value specified.");
@@ -854,11 +968,11 @@ int user_options_sanity (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
-  if (user_options->markov_hcstat != NULL)
+  if (user_options->markov_hcstat2 != NULL)
   {
-    if (strlen (user_options->markov_hcstat) == 0)
+    if (strlen (user_options->markov_hcstat2) == 0)
     {
-      event_log_error (hashcat_ctx, "Invalid --markov-hcstat value - must not be empty.");
+      event_log_error (hashcat_ctx, "Invalid --markov-hcstat2 value - must not be empty.");
 
       return -1;
     }
@@ -944,6 +1058,22 @@ int user_options_sanity (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
+  #ifdef WITH_BRAIN
+  if ((user_options->brain_client == true) && (user_options->remove == true))
+  {
+    event_log_error (hashcat_ctx, "Using --remove is not allowed if --brain-client is used.");
+
+    return -1;
+  }
+
+  if ((user_options->brain_client == true) && (user_options->potfile_disable == true))
+  {
+    event_log_error (hashcat_ctx, "Using --potfile-disable is not allowed if --brain-client is used.");
+
+    return -1;
+  }
+  #endif
+
   // custom charset checks
 
   if ((user_options->custom_charset_1 != NULL)
@@ -960,7 +1090,7 @@ int user_options_sanity (hashcat_ctx_t *hashcat_ctx)
 
     if (user_options->attack_mode == ATTACK_MODE_COMBI)
     {
-      event_log_error (hashcat_ctx, "Custom charsets re not supported in attack mode 1 (combination).");
+      event_log_error (hashcat_ctx, "Custom charsets are not supported in attack mode 1 (combination).");
 
       return -1;
     }
@@ -1002,6 +1132,12 @@ int user_options_sanity (hashcat_ctx_t *hashcat_ctx)
   {
     show_error = false;
   }
+  #ifdef WITH_BRAIN
+  else if (user_options->brain_server == true)
+  {
+    show_error = false;
+  }
+  #endif
   else if (user_options->benchmark == true)
   {
     if (user_options->hc_argc == 0)
@@ -1113,6 +1249,27 @@ int user_options_sanity (hashcat_ctx_t *hashcat_ctx)
       {
         show_error = false;
       }
+
+      if (user_options->hc_argc == 1)
+      {
+        // stdin mode
+
+        #ifdef WITH_BRAIN
+        if (user_options->brain_client == true)
+        {
+          event_log_error (hashcat_ctx, "Use of --brain-client is not possible in stdin mode.");
+
+          return -1;
+        }
+        #endif
+
+        if (user_options->slow_candidates == true)
+        {
+          event_log_error (hashcat_ctx, "Use of --slow-candidates is not possible in stdin mode.");
+
+          return -1;
+        }
+      }
     }
     else if (user_options->attack_mode == ATTACK_MODE_COMBI)
     {
@@ -1211,13 +1368,15 @@ void user_options_preprocess (hashcat_ctx_t *hashcat_ctx)
 {
   user_options_t *user_options = hashcat_ctx->user_options;
 
-  #if !defined (WITH_HWMON)
-  user_options->gpu_temp_disable = true;
-  user_options->gpu_temp_abort   = 0;
-  #endif // WITH_HWMON
-
   // some options can influence or overwrite other options
 
+  #ifdef WITH_BRAIN
+  if (user_options->brain_client == true)
+  {
+    user_options->slow_candidates = true;
+  }
+  #endif
+
   if (user_options->example_hashes  == true
    || user_options->opencl_info     == true
    || user_options->keyspace        == true
@@ -1237,6 +1396,11 @@ void user_options_preprocess (hashcat_ctx_t *hashcat_ctx)
     user_options->show                = false;
     user_options->status              = false;
     user_options->status_timer        = 0;
+    user_options->bitmap_min          = 1;
+    user_options->bitmap_max          = 1;
+    #ifdef WITH_BRAIN
+    user_options->brain_client        = false;
+    #endif
   }
 
   if (user_options->benchmark == true)
@@ -1256,6 +1420,11 @@ void user_options_preprocess (hashcat_ctx_t *hashcat_ctx)
     user_options->speed_only          = true;
     user_options->status              = false;
     user_options->status_timer        = 0;
+    user_options->bitmap_min          = 1;
+    user_options->bitmap_max          = 1;
+    #ifdef WITH_BRAIN
+    user_options->brain_client        = false;
+    #endif
 
     if (user_options->workload_profile_chgd == false)
     {
@@ -1279,6 +1448,11 @@ void user_options_preprocess (hashcat_ctx_t *hashcat_ctx)
     user_options->quiet = true;
   }
 
+  if (user_options->slow_candidates == true)
+  {
+    user_options->opencl_vector_width = 1;
+  }
+
   if (user_options->stdout_flag == true)
   {
     user_options->force               = true;
@@ -1370,6 +1544,10 @@ void user_options_preprocess (hashcat_ctx_t *hashcat_ctx)
     user_options->segment_size *= (1024 * 1024);
   }
 
+  #if !defined (WITH_HWMON)
+  user_options->gpu_temp_disable = true;
+  #endif // WITH_HWMON
+
   if (user_options->gpu_temp_disable == true)
   {
     user_options->gpu_temp_abort  = 0;
@@ -1643,27 +1821,35 @@ u64 user_options_extra_amplifier (hashcat_ctx_t *hashcat_ctx)
   const combinator_ctx_t     *combinator_ctx     = hashcat_ctx->combinator_ctx;
   const mask_ctx_t           *mask_ctx           = hashcat_ctx->mask_ctx;
   const straight_ctx_t       *straight_ctx       = hashcat_ctx->straight_ctx;
+  const user_options_t       *user_options       = hashcat_ctx->user_options;
   const user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
-  if (user_options_extra->attack_kern == ATTACK_KERN_STRAIGHT)
+  if (user_options->slow_candidates == true)
   {
-    if (straight_ctx->kernel_rules_cnt)
-    {
-      return straight_ctx->kernel_rules_cnt;
-    }
+    return 1;
   }
-  else if (user_options_extra->attack_kern == ATTACK_KERN_COMBI)
+  else
   {
-    if (combinator_ctx->combs_cnt)
+    if (user_options_extra->attack_kern == ATTACK_KERN_STRAIGHT)
     {
-      return combinator_ctx->combs_cnt;
+      if (straight_ctx->kernel_rules_cnt)
+      {
+        return straight_ctx->kernel_rules_cnt;
+      }
     }
-  }
-  else if (user_options_extra->attack_kern == ATTACK_KERN_BF)
-  {
-    if (mask_ctx->bfs_cnt)
+    else if (user_options_extra->attack_kern == ATTACK_KERN_COMBI)
     {
-      return mask_ctx->bfs_cnt;
+      if (combinator_ctx->combs_cnt)
+      {
+        return combinator_ctx->combs_cnt;
+      }
+    }
+    else if (user_options_extra->attack_kern == ATTACK_KERN_BF)
+    {
+      if (mask_ctx->bfs_cnt)
+      {
+        return mask_ctx->bfs_cnt;
+      }
     }
   }
 
@@ -1682,6 +1868,51 @@ int user_options_check_files (hashcat_ctx_t *hashcat_ctx)
   user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
   user_options_t       *user_options       = hashcat_ctx->user_options;
 
+  // brain
+
+  #ifdef WITH_BRAIN
+  #if defined (_WIN)
+  if ((user_options->brain_client == true) || (user_options->brain_server == true))
+  {
+    WSADATA wsaData;
+
+    WORD wVersionRequested = MAKEWORD (2,2);
+
+    const int iResult = WSAStartup (wVersionRequested, &wsaData);
+
+    if (iResult != NO_ERROR)
+    {
+      fprintf (stderr, "WSAStartup: %s\n", strerror (errno));
+
+      return -1;
+    }
+  }
+  #endif
+
+  if (user_options->brain_host)
+  {
+    struct addrinfo hints;
+
+    memset (&hints, 0, sizeof (hints));
+
+    hints.ai_family = AF_INET;
+    hints.ai_socktype = SOCK_STREAM;
+
+    struct addrinfo *address_info = NULL;
+
+    const int rc_getaddrinfo = getaddrinfo (user_options->brain_host, NULL, &hints, &address_info);
+
+    if (rc_getaddrinfo != 0)
+    {
+      fprintf (stderr, "%s: %s\n", user_options->brain_host, gai_strerror (rc_getaddrinfo));
+
+      return -1;
+    }
+
+    freeaddrinfo (address_info);
+  }
+  #endif
+
   // common folders
 
   #if defined (_WIN)
@@ -2259,6 +2490,9 @@ void user_options_logger (hashcat_ctx_t *hashcat_ctx)
   logfile_ctx_t  *logfile_ctx  = hashcat_ctx->logfile_ctx;
 
   logfile_top_char   (user_options->separator);
+  #ifdef WITH_BRAIN
+  logfile_top_string (user_options->brain_session_whitelist);
+  #endif
   logfile_top_string (user_options->cpu_affinity);
   logfile_top_string (user_options->custom_charset_1);
   logfile_top_string (user_options->custom_charset_2);
@@ -2268,7 +2502,7 @@ void user_options_logger (hashcat_ctx_t *hashcat_ctx)
   logfile_top_string (user_options->encoding_from);
   logfile_top_string (user_options->encoding_to);
   logfile_top_string (user_options->induction_dir);
-  logfile_top_string (user_options->markov_hcstat);
+  logfile_top_string (user_options->markov_hcstat2);
   logfile_top_string (user_options->opencl_devices);
   logfile_top_string (user_options->opencl_device_types);
   logfile_top_string (user_options->opencl_platforms);
@@ -2282,6 +2516,9 @@ void user_options_logger (hashcat_ctx_t *hashcat_ctx)
   logfile_top_string (user_options->session);
   logfile_top_string (user_options->truecrypt_keyfiles);
   logfile_top_string (user_options->veracrypt_keyfiles);
+  #ifdef WITH_BRAIN
+  logfile_top_string (user_options->brain_host);
+  #endif
   logfile_top_uint64 (user_options->limit);
   logfile_top_uint64 (user_options->skip);
   logfile_top_uint   (user_options->attack_mode);
@@ -2337,6 +2574,7 @@ void user_options_logger (hashcat_ctx_t *hashcat_ctx)
   logfile_top_uint   (user_options->scrypt_tmto);
   logfile_top_uint   (user_options->segment_size);
   logfile_top_uint   (user_options->self_test_disable);
+  logfile_top_uint   (user_options->slow_candidates);
   logfile_top_uint   (user_options->show);
   logfile_top_uint   (user_options->speed_only);
   logfile_top_uint   (user_options->status);
@@ -2347,4 +2585,11 @@ void user_options_logger (hashcat_ctx_t *hashcat_ctx)
   logfile_top_uint   (user_options->veracrypt_pim);
   logfile_top_uint   (user_options->version);
   logfile_top_uint   (user_options->workload_profile);
+  #ifdef WITH_BRAIN
+  logfile_top_uint   (user_options->brain_client);
+  logfile_top_uint   (user_options->brain_client_features);
+  logfile_top_uint   (user_options->brain_server);
+  logfile_top_uint   (user_options->brain_port);
+  logfile_top_uint   (user_options->brain_session);
+  #endif
 }
